@@ -1,66 +1,120 @@
 /** -*- c++ -*-
  * progressdialog.cpp
  *
- *  Copyright (c) 2004 Till Adam <adam@kde.org>,
- *                     David Faure <faure@kde.org>
+ *  SPDX-FileCopyrightText: 2004 Till Adam <adam@kde.org>
+ *  SPDX-FileCopyrightText: 2004 David Faure <faure@kde.org>
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; version 2 of the License
- *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
- *
- *  In addition, as a special exception, the copyright holders give
- *  permission to link the code of this program with any edition of
- *  the Qt library by Trolltech AS, Norway (or with modified versions
- *  of Qt that use the same license as Qt), and distribute linked
- *  combinations including the two.  You must obey the GNU General
- *  Public License in all respects for all of the code used other than
- *  Qt.  If you modify this file, you may extend this exception to
- *  your version of the file, but you are not obligated to do so.  If
- *  you do not wish to do so, delete this exception statement from
- *  your version.
+ *  SPDX-License-Identifier: GPL-2.0-or-later
  */
 
 #include "progressdialog.h"
 #include "ssllabel.h"
 
-#include <QHBoxLayout>
 #include <KLocalizedString>
+#include <QHBoxLayout>
 
 #include <QCloseEvent>
 #include <QFrame>
 #include <QLabel>
+#include <QLayout>
 #include <QObject>
 #include <QProgressBar>
 #include <QPushButton>
 #include <QScrollBar>
 #include <QTimer>
-#include <QLayout>
 #include <QVBoxLayout>
+#include <chrono>
 
-namespace KPIM
-{
+using namespace std::chrono_literals;
 
+using namespace KPIM;
 static const int MAX_LABEL_WIDTH = 650;
 
-class TransactionItem;
-
-TransactionItemView::TransactionItemView(QWidget *parent, const char *name)
-    : QScrollArea(parent)
+class KPIM::OverlayWidgetPrivate
 {
-    setObjectName(QLatin1String(name));
+public:
+    OverlayWidgetPrivate() = default;
+
+    QWidget *mAlignWidget = nullptr;
+};
+
+OverlayWidget::OverlayWidget(QWidget *alignWidget, QWidget *parent)
+    : QFrame(parent)
+    , d(new KPIM::OverlayWidgetPrivate)
+{
+    setAlignWidget(alignWidget);
+    setLayout(new QHBoxLayout(this));
+}
+
+OverlayWidget::~OverlayWidget() = default;
+
+QWidget *OverlayWidget::alignWidget() const
+{
+    return d->mAlignWidget;
+}
+
+void OverlayWidget::reposition()
+{
+    if (!d->mAlignWidget) {
+        return;
+    }
+    // p is in the alignWidget's coordinates
+    QPoint p;
+    // We are always above the alignWidget, right-aligned with it for
+    // LTR locales, and left-aligned for RTL locales (default value=0).
+    if (layoutDirection() == Qt::LeftToRight) {
+        p.setX(d->mAlignWidget->width() - width());
+    }
+    p.setY(-height());
+    // Position in the toplevelwidget's coordinates
+    QPoint pTopLevel = d->mAlignWidget->mapTo(topLevelWidget(), p);
+    // Position in the widget's parentWidget coordinates
+    QPoint pParent = parentWidget()->mapFrom(topLevelWidget(), pTopLevel);
+    // Move 'this' to that position.
+    move(pParent);
+}
+
+void OverlayWidget::setAlignWidget(QWidget *w)
+{
+    if (w == d->mAlignWidget) {
+        return;
+    }
+
+    if (d->mAlignWidget) {
+        d->mAlignWidget->removeEventFilter(this);
+    }
+
+    d->mAlignWidget = w;
+
+    if (d->mAlignWidget) {
+        d->mAlignWidget->installEventFilter(this);
+    }
+
+    reposition();
+}
+
+bool OverlayWidget::eventFilter(QObject *o, QEvent *e)
+{
+    if (o == d->mAlignWidget && (e->type() == QEvent::Move || e->type() == QEvent::Resize)) {
+        reposition();
+    }
+    return QFrame::eventFilter(o, e);
+}
+
+void OverlayWidget::resizeEvent(QResizeEvent *ev)
+{
+    reposition();
+    QFrame::resizeEvent(ev);
+}
+
+TransactionItemView::TransactionItemView(QWidget *parent, const QString &name)
+    : QScrollArea(parent)
+    , mBigBox(new QWidget(this))
+{
+    setObjectName(name);
     setFrameStyle(NoFrame);
-    mBigBox = new QWidget(this);
-    QVBoxLayout *mBigBoxVBoxLayout = new QVBoxLayout(mBigBox);
-    mBigBoxVBoxLayout->setMargin(0);
+    auto mBigBoxVBoxLayout = new QVBoxLayout(mBigBox);
+    mBigBoxVBoxLayout->setContentsMargins(0, 0, 0, 0);
     setWidget(mBigBox);
     setWidgetResizable(true);
     setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Fixed);
@@ -68,7 +122,7 @@ TransactionItemView::TransactionItemView(QWidget *parent, const char *name)
 
 TransactionItem *TransactionItemView::addTransactionItem(ProgressItem *item, bool first)
 {
-    TransactionItem *ti = new TransactionItem(mBigBox, item, first);
+    auto ti = new TransactionItem(mBigBox, item, first);
     mBigBox->layout()->addWidget(ti);
 
     resize(mBigBox->width(), mBigBox->height());
@@ -81,7 +135,7 @@ void TransactionItemView::resizeEvent(QResizeEvent *event)
     // Tell the layout in the parent (progressdialog) that our size changed
     updateGeometry();
 
-    QSize sz = parentWidget()->sizeHint();
+    const QSize sz = parentWidget()->sizeHint();
     int currentWidth = parentWidget()->width();
 
     // Don't resize to sz.width() every time when it only reduces a little bit
@@ -100,11 +154,11 @@ QSize TransactionItemView::sizeHint() const
 
 QSize TransactionItemView::minimumSizeHint() const
 {
-    int f = 2 * frameWidth();
+    const int f = 2 * frameWidth();
     // Make room for a vertical scrollbar in all cases, to avoid a horizontal one
-    int vsbExt = verticalScrollBar()->sizeHint().width();
-    int minw = topLevelWidget()->width() / 3;
-    int maxh = topLevelWidget()->height() / 2;
+    const int vsbExt = verticalScrollBar()->sizeHint().width();
+    const int minw = topLevelWidget()->width() / 3;
+    const int maxh = topLevelWidget()->height() / 2;
     QSize sz(mBigBox->minimumSizeHint());
     sz.setWidth(qMax(sz.width(), minw) + f + vsbExt);
     sz.setHeight(qMin(sz.height(), maxh) + f);
@@ -113,21 +167,21 @@ QSize TransactionItemView::minimumSizeHint() const
 
 void TransactionItemView::slotLayoutFirstItem()
 {
-    //This slot is called whenever a TransactionItem is deleted, so this is a
-    //good place to call updateGeometry(), so our parent takes the new size
-    //into account and resizes.
+    // This slot is called whenever a TransactionItem is deleted, so this is a
+    // good place to call updateGeometry(), so our parent takes the new size
+    // into account and resizes.
     updateGeometry();
 
     /*
      The below relies on some details in Qt's behaviour regarding deleting
      objects. This slot is called from the destroyed signal of an item just
-     going away. That item is at that point still in the  list of chilren, but
+     going away. That item is at that point still in the  list of children, but
      since the vtable is already gone, it will have type QObject. The first
-     one with both the right name and the right class therefor is what will
+     one with both the right name and the right class therefore is what will
      be the first item very shortly. That's the one we want to remove the
      hline for.
     */
-    TransactionItem *ti = mBigBox->findChild<KPIM::TransactionItem *>(QStringLiteral("TransactionItem"));
+    auto ti = mBigBox->findChild<KPIM::TransactionItem *>(QStringLiteral("TransactionItem"));
     if (ti) {
         ti->hideHLine();
     }
@@ -135,15 +189,13 @@ void TransactionItemView::slotLayoutFirstItem()
 
 // ----------------------------------------------------------------------------
 
-TransactionItem::TransactionItem(QWidget *parent,
-                                 ProgressItem *item, bool first)
-    : QWidget(parent), mCancelButton(Q_NULLPTR), mItem(item)
-
+TransactionItem::TransactionItem(QWidget *parent, ProgressItem *item, bool first)
+    : QWidget(parent)
+    , mItem(item)
 {
-    QVBoxLayout *vboxLayout = new QVBoxLayout;
+    auto vboxLayout = new QVBoxLayout(this);
     vboxLayout->setSpacing(2);
-    vboxLayout->setMargin(2);
-    setLayout(vboxLayout);
+    vboxLayout->setContentsMargins(2, 2, 2, 2);
     setSizePolicy(QSizePolicy(QSizePolicy::Preferred, QSizePolicy::Fixed));
 
     mFrame = new QFrame(this);
@@ -152,14 +204,13 @@ TransactionItem::TransactionItem(QWidget *parent,
     mFrame->show();
     layout()->addWidget(mFrame);
 
-    QWidget *h = new QWidget(this);
-    QHBoxLayout *hHBoxLayout = new QHBoxLayout(h);
-    hHBoxLayout->setMargin(0);
+    auto h = new QWidget(this);
+    auto hHBoxLayout = new QHBoxLayout(h);
+    hHBoxLayout->setContentsMargins(0, 0, 0, 0);
     hHBoxLayout->setSpacing(5);
     layout()->addWidget(h);
 
-    mItemLabel =
-        new QLabel(fontMetrics().elidedText(item->label(), Qt::ElideRight, MAX_LABEL_WIDTH), h);
+    mItemLabel = new QLabel(fontMetrics().elidedText(item->label(), Qt::ElideRight, MAX_LABEL_WIDTH), h);
     h->layout()->addWidget(mItemLabel);
     h->setSizePolicy(QSizePolicy(QSizePolicy::Preferred, QSizePolicy::Fixed));
 
@@ -173,14 +224,13 @@ TransactionItem::TransactionItem(QWidget *parent,
         mCancelButton = new QPushButton(QIcon::fromTheme(QStringLiteral("dialog-cancel")), QString(), h);
         hHBoxLayout->addWidget(mCancelButton);
         mCancelButton->setToolTip(i18n("Cancel this operation."));
-        connect(mCancelButton, &QAbstractButton::clicked,
-                this, &TransactionItem::slotItemCanceled);
+        connect(mCancelButton, &QAbstractButton::clicked, this, &TransactionItem::slotItemCanceled);
         h->layout()->addWidget(mCancelButton);
     }
 
     h = new QWidget(this);
     hHBoxLayout = new QHBoxLayout(h);
-    hHBoxLayout->setMargin(0);
+    hHBoxLayout->setContentsMargins(0, 0, 0, 0);
     hHBoxLayout->setSpacing(5);
     h->setSizePolicy(QSizePolicy(QSizePolicy::Preferred, QSizePolicy::Fixed));
     layout()->addWidget(h);
@@ -191,8 +241,7 @@ TransactionItem::TransactionItem(QWidget *parent,
     mItemStatus = new QLabel(h);
     hHBoxLayout->addWidget(mItemStatus);
     mItemStatus->setTextFormat(Qt::RichText);
-    mItemStatus->setText(
-        fontMetrics().elidedText(item->status(), Qt::ElideRight, MAX_LABEL_WIDTH));
+    mItemStatus->setText(fontMetrics().elidedText(item->status(), Qt::ElideRight, MAX_LABEL_WIDTH));
     h->layout()->addWidget(mItemStatus);
     setCryptoStatus(item->cryptoStatus());
     if (first) {
@@ -200,9 +249,7 @@ TransactionItem::TransactionItem(QWidget *parent,
     }
 }
 
-TransactionItem::~TransactionItem()
-{
-}
+KPIM::TransactionItem::~TransactionItem() = default;
 
 void TransactionItem::hideHLine()
 {
@@ -254,43 +301,39 @@ void TransactionItem::slotItemCanceled()
 
 void TransactionItem::addSubTransaction(ProgressItem *item)
 {
-    Q_UNUSED(item);
+    Q_UNUSED(item)
 }
 
 // ---------------------------------------------------------------------------
 
 ProgressDialog::ProgressDialog(QWidget *alignWidget, QWidget *parent)
-    : OverlayWidget(alignWidget, parent),
-      mShowTypeProgressItem(0),
-      mWasLastShown(false)
+    : OverlayWidget(alignWidget, parent)
 {
-    setFrameStyle(QFrame::Panel | QFrame::Sunken);   // QFrame
+    // Qt Bug: Sunken is not applied for RTL layouts correctly (is not mirrored).
+    // For now let's just use Plain, which is fine for this.
+    if (layoutDirection() == Qt::LeftToRight) {
+        setFrameStyle(QFrame::Panel | QFrame::Sunken); // QFrame
+    } else {
+        setFrameStyle(QFrame::Panel | QFrame::Plain); // QFrame
+    }
 
     setAutoFillBackground(true);
 
-    mScrollView = new TransactionItemView(this, "ProgressScrollView");
+    mScrollView = new TransactionItemView(this, QStringLiteral("ProgressScrollView"));
     layout()->addWidget(mScrollView);
     /*
-    * Get the singleton ProgressManager item which will inform us of
-    * appearing and vanishing items.
-    */
+     * Get the singleton ProgressManager item which will inform us of
+     * appearing and vanishing items.
+     */
     ProgressManager *pm = ProgressManager::instance();
-    connect(pm, &ProgressManager::progressItemAdded,
-            this, &ProgressDialog::slotTransactionAdded);
-    connect(pm, &ProgressManager::progressItemCompleted,
-            this, &ProgressDialog::slotTransactionCompleted);
-    connect(pm, &ProgressManager::progressItemProgress,
-            this, &ProgressDialog::slotTransactionProgress);
-    connect(pm, &ProgressManager::progressItemStatus,
-            this, &ProgressDialog::slotTransactionStatus);
-    connect(pm, &ProgressManager::progressItemLabel,
-            this, &ProgressDialog::slotTransactionLabel);
-    connect(pm, &ProgressManager::progressItemCryptoStatus,
-            this, &ProgressDialog::slotTransactionCryptoStatus);
-    connect(pm, &ProgressManager::progressItemUsesBusyIndicator,
-            this, &ProgressDialog::slotTransactionUsesBusyIndicator);
-    connect(pm, &ProgressManager::showProgressDialog,
-            this, &ProgressDialog::slotShow);
+    connect(pm, &ProgressManager::progressItemAdded, this, &ProgressDialog::slotTransactionAdded);
+    connect(pm, &ProgressManager::progressItemCompleted, this, &ProgressDialog::slotTransactionCompleted);
+    connect(pm, &ProgressManager::progressItemProgress, this, &ProgressDialog::slotTransactionProgress);
+    connect(pm, &ProgressManager::progressItemStatus, this, &ProgressDialog::slotTransactionStatus);
+    connect(pm, &ProgressManager::progressItemLabel, this, &ProgressDialog::slotTransactionLabel);
+    connect(pm, &ProgressManager::progressItemCryptoStatus, this, &ProgressDialog::slotTransactionCryptoStatus);
+    connect(pm, &ProgressManager::progressItemUsesBusyIndicator, this, &ProgressDialog::slotTransactionUsesBusyIndicator);
+    connect(pm, &ProgressManager::showProgressDialog, this, &ProgressDialog::slotShow);
 }
 
 void ProgressDialog::closeEvent(QCloseEvent *e)
@@ -299,6 +342,11 @@ void ProgressDialog::closeEvent(QCloseEvent *e)
     hide();
 }
 
+bool ProgressDialog::wasLastShown() const
+{
+    return mWasLastShown;
+}
+
 /*
  *  Destructor
  */
@@ -316,8 +364,7 @@ void ProgressDialog::slotTransactionAdded(ProgressItem *item)
 {
     if (item->typeProgressItem() == mShowTypeProgressItem) {
         if (item->parent()) {
-            if (mTransactionsToListviewItems.contains(item->parent())) {
-                TransactionItem *parent = mTransactionsToListviewItems[ item->parent() ];
+            if (TransactionItem *parent = mTransactionsToListviewItems.value(item->parent())) {
                 parent->addSubTransaction(item);
             }
         } else {
@@ -327,7 +374,7 @@ void ProgressDialog::slotTransactionAdded(ProgressItem *item)
                 mTransactionsToListviewItems.insert(item, ti);
             }
             if (first && mWasLastShown) {
-                QTimer::singleShot(1000, this, &ProgressDialog::slotShow);
+                QTimer::singleShot(1s, this, &ProgressDialog::slotShow);
             }
         }
     }
@@ -335,18 +382,16 @@ void ProgressDialog::slotTransactionAdded(ProgressItem *item)
 
 void ProgressDialog::slotTransactionCompleted(ProgressItem *item)
 {
-    if (mTransactionsToListviewItems.contains(item)) {
-        TransactionItem *ti = mTransactionsToListviewItems[ item ];
+    if (TransactionItem *ti = mTransactionsToListviewItems.value(item)) {
         mTransactionsToListviewItems.remove(item);
         ti->setItemComplete();
-        QTimer::singleShot(3000, ti, &QObject::deleteLater);
+        QTimer::singleShot(3s, ti, &QObject::deleteLater);
         // see the slot for comments as to why that works
-        connect(ti, &QObject::destroyed,
-                mScrollView, &TransactionItemView::slotLayoutFirstItem);
+        connect(ti, &QObject::destroyed, mScrollView, &TransactionItemView::slotLayoutFirstItem);
     }
     // This was the last item, hide.
     if (mTransactionsToListviewItems.empty()) {
-        QTimer::singleShot(3000, this, &ProgressDialog::slotHide);
+        QTimer::singleShot(3s, this, &ProgressDialog::slotHide);
     }
 }
 
@@ -354,46 +399,37 @@ void ProgressDialog::slotTransactionCanceled(ProgressItem *)
 {
 }
 
-void ProgressDialog::slotTransactionProgress(ProgressItem *item,
-        unsigned int progress)
+void ProgressDialog::slotTransactionProgress(ProgressItem *item, unsigned int progress)
 {
-    if (mTransactionsToListviewItems.contains(item)) {
-        TransactionItem *ti = mTransactionsToListviewItems[ item ];
+    if (TransactionItem *ti = mTransactionsToListviewItems.value(item)) {
         ti->setProgress(progress);
     }
 }
 
-void ProgressDialog::slotTransactionStatus(ProgressItem *item,
-        const QString &status)
+void ProgressDialog::slotTransactionStatus(ProgressItem *item, const QString &status)
 {
-    if (mTransactionsToListviewItems.contains(item)) {
-        TransactionItem *ti = mTransactionsToListviewItems[ item ];
+    if (TransactionItem *ti = mTransactionsToListviewItems.value(item)) {
         ti->setStatus(status);
     }
 }
 
-void ProgressDialog::slotTransactionLabel(ProgressItem *item,
-        const QString &label)
+void ProgressDialog::slotTransactionLabel(ProgressItem *item, const QString &label)
 {
-    if (mTransactionsToListviewItems.contains(item)) {
-        TransactionItem *ti = mTransactionsToListviewItems[ item ];
+    if (TransactionItem *ti = mTransactionsToListviewItems.value(item)) {
         ti->setLabel(label);
     }
 }
 
-void ProgressDialog::slotTransactionCryptoStatus(ProgressItem *item,
-        KPIM::ProgressItem::CryptoStatus value)
+void ProgressDialog::slotTransactionCryptoStatus(ProgressItem *item, KPIM::ProgressItem::CryptoStatus value)
 {
-    if (mTransactionsToListviewItems.contains(item)) {
-        TransactionItem *ti = mTransactionsToListviewItems[ item ];
+    if (TransactionItem *ti = mTransactionsToListviewItems.value(item)) {
         ti->setCryptoStatus(value);
     }
 }
 
 void ProgressDialog::slotTransactionUsesBusyIndicator(KPIM::ProgressItem *item, bool value)
 {
-    if (mTransactionsToListviewItems.contains(item)) {
-        TransactionItem *ti = mTransactionsToListviewItems[ item ];
+    if (TransactionItem *ti = mTransactionsToListviewItems.value(item)) {
         if (value) {
             ti->setTotalSteps(0);
         } else {
@@ -430,15 +466,15 @@ void ProgressDialog::setVisible(bool b)
 void ProgressDialog::slotToggleVisibility()
 {
     /* Since we are only hiding with a timeout, there is a short period of
-    * time where the last item is still visible, but clicking on it in
-    * the statusbarwidget should not display the dialog, because there
-    * are no items to be shown anymore. Guard against that.
-    */
-    mWasLastShown = isHidden();
+     * time where the last item is still visible, but clicking on it in
+     * the statusbarwidget should not display the dialog, because there
+     * are no items to be shown anymore. Guard against that.
+     */
     if (!isHidden() || !mTransactionsToListviewItems.isEmpty()) {
-        setVisible(isHidden());
+        const bool showNow = isHidden();
+        setVisible(showNow);
+        mWasLastShown = showNow;
     }
 }
 
-}
-
+#include "moc_progressdialog.cpp"
